@@ -279,6 +279,9 @@ static int login(AFPObj *obj, struct passwd *pwd, void (*logout)(void), int expi
     obj->uid = pwd->pw_uid;
     obj->euid = geteuid();
 
+    /* report to parent */
+    ipc_child_write(obj->ipc_fd, IPC_LOGINDONE, 0, "");
+
     /* pam_umask or similar might have changed our umask */
     (void)umask(obj->options.umask);
 
@@ -334,7 +337,7 @@ int afp_zzz(AFPObj *obj, char *ibuf, size_t ibuflen, char *rbuf, size_t *rbuflen
 
     /*
      * According to AFP 3.3 spec we should not return anything,
-     * but eg 10.5.8 server still returns the numbers of hours
+     * but e.g. 10.5.8 server still returns the numbers of hours
      * the server is keeping the sessino (ie max sleeptime).
      */
     data = obj->options.sleep / 120; /* hours */
