@@ -1,7 +1,3 @@
-/*
- * $Id: cnid_cdb_get.c,v 1.1.4.2.2.2 2005/09/27 10:40:41 didg Exp $
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
@@ -10,8 +6,7 @@
 #include "cnid_cdb_private.h"
 
 /* Return CNID for a given did/name. */
-cnid_t cnid_cdb_get(struct _cnid_db *cdb, const cnid_t did, char *name,
-                const size_t len)
+cnid_t cnid_cdb_get(struct _cnid_db *cdb, cnid_t did, const char *name, size_t len)
 {
     char start[CNID_DID_LEN + MAXPATHLEN + 1], *buf;
     CNID_private *db;
@@ -19,7 +14,7 @@ cnid_t cnid_cdb_get(struct _cnid_db *cdb, const cnid_t did, char *name,
     cnid_t id;
     int rc;
 
-    if (!cdb || !(db = cdb->_private) || (len > MAXPATHLEN)) {
+    if (!cdb || !(db = cdb->cnid_db_private) || (len > MAXPATHLEN)) {
         return 0;
     }
 
@@ -46,7 +41,7 @@ cnid_t cnid_cdb_get(struct _cnid_db *cdb, const cnid_t did, char *name,
 
     memcpy(&id, data.data, sizeof(id));
 #ifdef DEBUG
-    LOG(log_info, logtype_default, "cnid_get: Returning CNID for %u, name %s as %u",
+    LOG(log_debug9, logtype_default, "cnid_get: Returning CNID for %u, name %s as %u",
         ntohl(did), name, ntohl(id));
 #endif
     return id;
