@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.  */
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+   02110-1301 USA.  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -45,7 +45,9 @@
 
 #include <atalk/ftw.h>
 
+#ifndef HAVE_MEMPCPY
 #define mempcpy(D, S, N) ((void *) ((char *) memcpy (D, S, N) + (N)))
+#endif
 
 #define NDEBUG 1
 #include <assert.h>
@@ -318,8 +320,8 @@ open_dir_stream (int *dfdp, struct ftw_data *data, struct dir_data *dirp)
             buf[actsize++] = '\0';
 
             /* Shrink the buffer to what we actually need.  */
-            if ( (data->dirstreams[data->actdir]->content =
-                realloc(buf, actsize)) == NULL)
+            data->dirstreams[data->actdir]->content = realloc (buf, actsize);
+            if (data->dirstreams[data->actdir]->content == NULL)
             {
                 int save_err = errno;
                 free (buf);
